import type {
  GetServerSidePropsContext,
  GetServerSidePropsResult,
  NextApiRequest,
  NextApiResponse,
} from "next";
import type { IronSession, IronSessionOptions } from "./";
import { getIronSession } from "./";

export interface NextApiRequestWithIronSession extends NextApiRequest {
  session: IronSession;
}

export declare type NextApiHandlerWithIronSession<T = any> = (
  req: NextApiRequestWithIronSession,
  res: NextApiResponse<T>,
) => void | Promise<void>;

interface NextGetSeverSideContextWithIronSession
  extends GetServerSidePropsContext {
  req: NextApiRequestWithIronSession;
}

export declare type NextGetServerSidePropsHandlerWithIronSession<P> = (
  context: NextGetSeverSideContextWithIronSession,
) => Promise<GetServerSidePropsResult<P>>;

export function withIronSessionApiRoute(
  handler: NextApiHandlerWithIronSession,
  options: IronSessionOptions,
): NextApiHandlerWithIronSession {
  return async function nextApiHandlerWrappedWithIronSession(req, res) {
    req.session = await getIronSession(req, res, options);
    return handler(req, res);
  };
}

export function withIronSessionSsr<P>(
  handler: NextGetServerSidePropsHandlerWithIronSession<P>,
  options: IronSessionOptions,
): NextGetServerSidePropsHandlerWithIronSession<P> {
  return async function nextGetServerSidePropsHandlerWrappedWithIronSession(
    context: NextGetSeverSideContextWithIronSession,
  ) {
    context.req.session = await getIronSession(
      context.req,
      context.res,
      options,
    );
    return handler(context);
  };
}
